@@ -169,7 +169,7 @@ void io_sys_init(void)
 	int i;
 	
 	/* How many processors on this machine? */
-	cpu_nr = get_cpu_nr(~0);
+	cpu_nr = get_cpu_nr(~0, FALSE);
 
 	/* Get number of CIFS directories in /proc/fs/cifs/Stats */
 	if ((cifs_nr = get_cifs_nr()) > 0) {
