@@ -15,20 +15,20 @@
  ***************************************************************************
  */
 
+extern int
+	get_cpu_nr(unsigned int, int);
+extern int
+	get_irqcpu_nr(char *, int, int);
+extern int
+	get_diskstats_dev_nr(int, int);
 extern int
 	get_irq_nr(void);
 extern int
 	get_serial_nr(void);
 extern int
 	get_iface_nr(void);
-extern int
-	get_diskstats_dev_nr(int, int);
 extern int
 	get_disk_nr(unsigned int);
-extern int
-	get_cpu_nr(unsigned int);
-extern int
-	get_irqcpu_nr(char *, int, int);
 extern int
 	get_freq_nr(void);
 extern int
