@@ -1,6 +1,6 @@
 /*
  * format.c: Output format definitions for sadf
- * (C) 2011 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 2011-2013 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -46,7 +46,7 @@ struct report_format hdr_fmt = {
  */
 struct report_format db_fmt = {
 	.id		= F_DB_OUTPUT,
-	.options	= FO_GROUPED_STATS + FO_TRUE_TIME + FO_HORIZONTALLY +
+	.options	= FO_GROUPED_STATS + FO_LOCAL_TIME + FO_HORIZONTALLY +
 			  FO_SEC_EPOCH + FO_FIELD_LIST,
 	.f_header	= NULL,
 	.f_statistics	= NULL,
@@ -60,7 +60,7 @@ struct report_format db_fmt = {
  */
 struct report_format ppc_fmt = {
 	.id		= F_PPC_OUTPUT,
-	.options	= FO_GROUPED_STATS + FO_TRUE_TIME + FO_SEC_EPOCH,
+	.options	= FO_GROUPED_STATS + FO_LOCAL_TIME + FO_SEC_EPOCH,
 	.f_header	= NULL,
 	.f_statistics	= NULL,
 	.f_timestamp	= NULL,
@@ -73,7 +73,7 @@ struct report_format ppc_fmt = {
  */
 struct report_format xml_fmt = {
 	.id		= F_XML_OUTPUT,
-	.options	= FO_HEADER_ONLY + FO_TRUE_TIME,
+	.options	= FO_HEADER_ONLY + FO_LOCAL_TIME,
 	.f_header	= print_xml_header,
 	.f_statistics	= print_xml_statistics,
 	.f_timestamp	= print_xml_timestamp,
@@ -86,7 +86,7 @@ struct report_format xml_fmt = {
  */
 struct report_format json_fmt = {
 	.id		= F_JSON_OUTPUT,
-	.options	= FO_HEADER_ONLY + FO_TRUE_TIME,
+	.options	= FO_HEADER_ONLY + FO_LOCAL_TIME,
 	.f_header	= print_json_header,
 	.f_statistics	= print_json_statistics,
 	.f_timestamp	= print_json_timestamp,
