@@ -251,9 +251,7 @@ void salloc_dev_list(int list_len)
  */
 void sfree_dev_list(void)
 {
-	if (st_dev_list) {
-		free(st_dev_list);
-	}
+	free(st_dev_list);
 }
 
 /*
@@ -305,7 +303,7 @@ void io_sys_init(void)
 	init_stats();
 
 	/* How many processors on this machine? */
-	cpu_nr = get_cpu_nr(~0);
+	cpu_nr = get_cpu_nr(~0, FALSE);
 
 	/* Get number of block devices and partitions in /proc/diskstats */
 	if ((iodev_nr = get_diskstats_dev_nr(CNT_PART, CNT_ALL_DEV)) > 0) {
@@ -402,21 +400,14 @@ void io_sys_free(void)
 	int i;
 	
 	for (i = 0; i < 2; i++) {
-
 		/* Free CPU structures */
-		if (st_cpu[i]) {
-			free(st_cpu[i]);
-		}
+		free(st_cpu[i]);
 
 		/* Free I/O device structures */
-		if (st_iodev[i]) {
-			free(st_iodev[i]);
-		}
+		free(st_iodev[i]);
 	}
 	
-	if (st_hdr_iodev) {
-		free(st_hdr_iodev);
-	}
+	free(st_hdr_iodev);
 }
 
 /*
