@@ -1,6 +1,6 @@
 /*
  * pr_stats.h: Include file used to display system statistics
- * (C) 1999-2011 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2013 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _PR_STATS_H
@@ -88,6 +88,8 @@ extern __print_funct_t print_pwr_wghfreq_stats
 	(struct activity *, int, int, unsigned long long);
 extern __print_funct_t print_pwr_usb_stats
 	(struct activity *, int, int, unsigned long long);
+extern __print_funct_t print_filesystem_stats
+	(struct activity *, int, int, unsigned long long);
 
 /* Functions used to display average statistics */
 extern __print_funct_t print_avg_memory_stats
@@ -112,5 +114,7 @@ extern __print_funct_t print_avg_huge_stats
 	(struct activity *, int, int, unsigned long long);
 extern __print_funct_t print_avg_pwr_usb_stats
 	(struct activity *, int, int, unsigned long long);
+extern __print_funct_t print_avg_filesystem_stats
+	(struct activity *, int, int, unsigned long long);
 
 #endif /* _PR_STATS_H */
