@@ -9,6 +9,7 @@
 #include <stdio.h>
 
 #include "common.h"
+#include "prealloc.h"
 #include "rd_stats.h"
 #include "rd_sensors.h"
 
@@ -176,6 +177,7 @@
 #define G_SNMP		0x04
 #define G_IPV6		0x08
 #define G_POWER		0x10
+#define G_XDISK		0x20
 
 /* sadc program */
 #define SADC		"sadc"
@@ -191,14 +193,6 @@
  */
 #define BITMAP_SIZE(m)	((((m) + 1) / 8) + 1)
 
-/* Pre-allocation constants */
-#define NR_IFACE_PREALLOC	2
-#define NR_SERIAL_PREALLOC	2
-#define NR_DISK_PREALLOC	3
-#define NR_FREQ_PREALLOC	0
-#define NR_USB_PREALLOC		5
-#define NR_FILESYSTEM_PREALLOC	3
-
 #define UTSNAME_LEN		65
 #define TIMESTAMP_LEN		16
 #define HEADER_LINE_LEN		512
@@ -802,6 +796,8 @@ extern unsigned int
 	check_net_dev_reg(struct activity *, int, int, unsigned int);
 extern unsigned int
 	check_net_edev_reg(struct activity *, int, int, unsigned int);
+extern double
+	compute_ifutil(struct stats_net_dev *, double, double);
 extern void
 	copy_structures(struct activity * [], unsigned int [],
 			struct record_header [], int, int);
