@@ -131,7 +131,7 @@ char *get_devname_from_sysfs(unsigned int major, unsigned int minor)
 	char *devname;
 	ssize_t r;
 
-	snprintf(link, 32, "%s/%d:%d", SYSFS_DEV_BLOCK, major, minor);
+	snprintf(link, 32, "%s/%u:%u", SYSFS_DEV_BLOCK, major, minor);
 
 	/* Get full path to device knowing its major and minor numbers */
 	r = readlink(link, target, PATH_MAX);
@@ -172,7 +172,7 @@ char *get_devname(unsigned int major, unsigned int minor, int pretty)
 	static char buf[32];
 	char *name;
 
-	snprintf(buf, 32, "dev%d-%d", major, minor);
+	snprintf(buf, 32, "dev%u-%u", major, minor);
 
 	if (!pretty)
 		return (buf);
@@ -311,7 +311,7 @@ int datecmp(struct tm *rectime, struct tstamp *tse)
 
 /*
  ***************************************************************************
- * Parse a time stamp entered on the command line (hh:mm:ss) and decode it.
+ * Parse a timestamp entered on the command line (hh:mm:ss) and decode it.
  *
  * IN:
  * @argv		Arguments list.
@@ -1664,3 +1664,42 @@ int parse_sa_P_opt(char *argv[], int *opt, unsigned int *flags, struct activity
 	return 0;
 }
 
+/*
+ ***************************************************************************
+ * Compute network interface utilization.
+ *
+ * IN:
+ * @st_net_dev	Structure with network interface stats.
+ * @rx		Number of bytes received per second.
+ * @tx		Number of bytes transmitted per second.
+ *
+ * RETURNS:
+ * NIC utilization (0-100%).
+ ***************************************************************************
+ */
+double compute_ifutil(struct stats_net_dev *st_net_dev, double rx, double tx)
+{
+	unsigned long long speed;
+	
+	if (st_net_dev->speed) {
+		
+		speed = st_net_dev->speed * 1000000;
+		
+		if (st_net_dev->duplex == C_DUPLEX_FULL) {
+			/* Full duplex */
+			if (rx > tx) {
+				return (rx * 800 / speed);
+			}
+			else {
+				return (tx * 800 / speed);
+			}
+		}
+		else {
+			/* Half duplex */
+			return ((rx + tx) * 800 / speed);
+		}
+	}
+	
+	return 0;
+}
+
