@@ -738,6 +738,7 @@ __print_funct_t xml_print_net_dev_stats(struct activity *a, int curr, int tab,
 {
 	int i, j;
 	struct stats_net_dev *sndc, *sndp;
+	double rxkb, txkb, ifutil;
 
 	if (!IS_SELECTED(a->options) || (a->nr <= 0))
 		goto close_xml_markup;
@@ -755,6 +756,10 @@ __print_funct_t xml_print_net_dev_stats(struct activity *a, int curr, int tab,
 		j = check_net_dev_reg(a, curr, !curr, i);
 		sndp = (struct stats_net_dev *) ((char *) a->buf[!curr] + j * a->msize);
 
+		rxkb = S_VALUE(sndp->rx_bytes, sndc->rx_bytes, itv);
+		txkb = S_VALUE(sndp->tx_bytes, sndc->tx_bytes, itv);
+		ifutil = compute_ifutil(sndc, rxkb, txkb);
+
 		xprintf(tab, "<net-dev iface=\"%s\" "
 			"rxpck=\"%.2f\" "
 			"txpck=\"%.2f\" "
@@ -762,15 +767,17 @@ __print_funct_t xml_print_net_dev_stats(struct activity *a, int curr, int tab,
 			"txkB=\"%.2f\" "
 			"rxcmp=\"%.2f\" "
 			"txcmp=\"%.2f\" "
-			"rxmcst=\"%.2f\"/>",
+			"rxmcst=\"%.2f\" "
+			"ifutil-percent=\"%.2f\"/>",
 			sndc->interface,
 			S_VALUE(sndp->rx_packets,    sndc->rx_packets,    itv),
 			S_VALUE(sndp->tx_packets,    sndc->tx_packets,    itv),
-			S_VALUE(sndp->rx_bytes,      sndc->rx_bytes,      itv) / 1024,
-			S_VALUE(sndp->tx_bytes,      sndc->tx_bytes,      itv) / 1024,
+			rxkb / 1024,
+			txkb / 1024,
 			S_VALUE(sndp->rx_compressed, sndc->rx_compressed, itv),
 			S_VALUE(sndp->tx_compressed, sndc->tx_compressed, itv),
-			S_VALUE(sndp->multicast,     sndc->multicast,     itv));
+			S_VALUE(sndp->multicast,     sndc->multicast,     itv),
+			ifutil);
 	}
 	tab--;
 
